#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Conbagui's  tutorials https://www.deviantart.com/conbagui/art/Create-a-Spiderman-Text-with-Gimp-2-10-806739567

# author: Pocholo
# date: 3/1/21

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.#


from gimpfu import *
import gettext
import os, sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def create_a_web_and_spiderman_text(filename, customtext, fontname, size):
          
    #Initiates the temporary state
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
    
    #Create the image    
    img = pdb.gimp_image_new(1920, 1080, RGB)
    
    #Open a photo
    imagLayer = pdb.gimp_file_load_layer(img, filename)
    pdb.gimp_image_add_layer(img, imagLayer, 0)
   
    #Center the image to the canvas and merge 
    imageW = pdb.gimp_drawable_width(imagLayer)
    imageH = pdb.gimp_drawable_height(imagLayer)
    pdb.gimp_layer_scale(imagLayer, imageW, imageH, TRUE)     
    offx = (imagLayer.width - imagLayer.width) / 2
    offy = (imagLayer.height - imagLayer.height) / 2
    pdb.gimp_layer_set_offsets(imagLayer, offx, offy)    
    pdb.gimp_image_set_active_layer(img, imagLayer)
    pdb.gimp_item_set_name(imagLayer, "Background") 
    pdb.gimp_layer_resize_to_image_size(imagLayer)
    
    #See process
    pdb.gimp_display_new(img)
    
    #Make the spider web       
    webLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Web", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, webLayer, 0)
    pdb.gimp_layer_add_alpha(webLayer)   
    pdb.gimp_drawable_fill(webLayer, FILL_TRANSPARENT)
    
    #Create the center spike
    pdb.script_fu_flowerspike_path(img, webLayer, "Center", 960, 540, 10, 16, 11.0, 1.0, 0, 1.0, 0, FALSE, (0, 0, 0), FALSE, (255, 255, 255))
	
    #Apply voronoi 
    vectors = pdb.gimp_image_get_active_vectors(img)
    pdb.plug_in_voronoi_path(img, vectors, FALSE)
    vectors = pdb.gimp_image_get_vectors_by_name(img, "Center")
        
    #Set the stroke brush and stroke the voronoi path
    pdb.gimp_context_set_brush("2. Hardness 100")
    pdb.gimp_context_set_brush_size(1)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.1)
    pdb.gimp_context_set_foreground((255, 255, 255))
    pdb.gimp_edit_stroke_vectors(webLayer, vectors)
                
    #Create the all the web paths with stroke
    pdb.script_fu_flowerspike_path(img, webLayer, "Path #1", 960, 540, 40, 16, 0, 0.9, 0, 0.9, 0, TRUE, (255, 255, 255), FALSE, (255, 255, 255))
    pdb.script_fu_flowerspike_path(img, webLayer, "Path #2", 960, 540, 100, 16, 0, 0.9, 0, 0.9, 0, TRUE, (255, 255, 255), FALSE, (255, 255, 255))
    pdb.script_fu_flowerspike_path(img, webLayer, "Path #3", 960, 540, 160, 16, 0, 0.9, 0, 0.9, 0, TRUE, (255, 255, 255), FALSE, (255, 255, 255))
    pdb.script_fu_flowerspike_path(img, webLayer, "Path #4", 960, 540, 220, 16, 0, 0.9, 0, 0.9, 0, TRUE, (255, 255, 255), FALSE, (255, 255, 255))
    pdb.script_fu_flowerspike_path(img, webLayer, "Path #5", 960, 540, 280, 16, 0, 0.9, 0, 0.9, 0, TRUE, (255, 255, 255), FALSE, (255, 255, 255))
    pdb.script_fu_flowerspike_path(img, webLayer, "Path #6", 960, 540, 340, 16, 0, 0.9, 0, 0.9, 0, TRUE, (255, 255, 255), FALSE, (255, 255, 255))
    pdb.script_fu_flowerspike_path(img, webLayer, "Path #7", 960, 540, 400, 16, 0, 0.9, 0, 0.9, 0, TRUE, (255, 255, 255), FALSE, (255, 255, 255))
    pdb.script_fu_flowerspike_path(img, webLayer, "Path #8", 960, 540, 460, 16, 0, 0.9, 0, 0.9, 0, TRUE, (255, 255, 255), FALSE, (255, 255, 255))
    pdb.script_fu_flowerspike_path(img, webLayer, "Path #8", 960, 540, 512, 16, 0, 0.9, 0, 0.9, 0, TRUE, (255, 255, 255), FALSE, (255, 255, 255))    
    pdb.script_fu_flowerspike_path(img, webLayer, "Path #8", 960, 540, 570, 16, 0, 0.9, 0, 0.9, 0, TRUE, (255, 255, 255), FALSE, (255, 255, 255))
    
    #Make web imperfect
    pdb.plug_in_whirl_pinch(img, webLayer, 10.0, 0, 0.300)
     
     #Create the text		
    textW = size/0.3
    textH = size/1 
    pdb.gimp_context_set_foreground((0, 0, 0))
    #textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 10, 1, size, PIXELS, fontname)
    textLayer = pdb.gimp_text_layer_new(img, customtext, fontname, size, PIXELS)
    pdb.gimp_image_add_layer(img, textLayer, 0)
    pdb.gimp_layer_add_alpha(textLayer)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 50)
    	
	#Resize text
    pdb.gimp_layer_scale (textLayer, textW, textH, TRUE)
			
	#Making the text centered	
    textLayer = pdb.gimp_image_get_active_layer(img)
    pdb.gimp_image_set_active_layer(img, textLayer)
    wh = pdb.gimp_drawable_width(textLayer)
    hh = pdb.gimp_drawable_height(textLayer)
    w = pdb.gimp_image_width(img)
    h = pdb.gimp_image_height(img)
    pdb.gimp_image_set_active_layer(img,textLayer)
    centerx=w/2
    centery=h/1.3
    adw=wh/2
    adh=hh/2
    sw=centerx - adw
    sh=centery - adh		
    pdb.gimp_layer_set_offsets(textLayer, sw, sh)
    pdb.gimp_layer_resize_to_image_size(textLayer)
      
    #Rename the text layer
    pdb.gimp_item_set_name(textLayer, "Text")
    pdb.gimp_layer_set_opacity(textLayer, 0)
       
    #Text drop shadow
    pdb.python_layerfx_drop_shadow(img, textLayer, 
                                       ( 0, 0, 0),   #color
                                               75,   #opacity
                                                0,   #contou - linear
                                                0,   #noise 
                                                3,   #mode for multiply 
                                                6,   #spread 
                                               25,   #size 
                                               55,   #offset_angle 
                                               35,   #offset_distance 
                                             TRUE,   #knockout 
                                            FALSE)   #merge
    
    #Text outer glow
    pdb.python_layerfx_outer_glow(img, textLayer, 
								       (0, 0, 0),	#color 
										  	 100,   #opacity
											   0,   #contou
											   0,   #noise
											   3,   #mode - multiply
											  60,   #spread 
											  30,   #size 
											TRUE,   #knockout 
										   FALSE)   #merge	
    
    #Text stroke    
    pdb.python_layerfx_stroke(img, textLayer, 
				                   (0, 0, 0),   #color
                                         100,	#opacity
										   0,	#mode - normal
									      17,	#size
									     100,	#position
									   FALSE)	#merge    
        
    #Pattern overlay		
    textstroke = pdb.gimp_image_get_layer_by_name(img, "Text-stroke")
    pdb.python_layerfx_pattern_overlay(img, textstroke, 
						              "Blue ovals.jpg",   #pattern
								                   100,   #opacity
							                         0,   #mode - normal
								                    17,   #scale
									                 0,   #interpolation
								                 FALSE)   #merge
    
    #Bevel and emboss
    textstrokepat = pdb.gimp_image_get_layer_by_name(img, "Text-stroke-pattern")
    pdb.python_layerfx_bevel_emboss(img, textstrokepat, 
												     1,   #style inner bevel
											        65,   #depth 
												     0,   #direction 
											        14,   #size 
											         3,   #soften
											        55,   #angle 
											        30,   #altitude 
											         0,   #gloss_contour - linear
										       "White",   #highlight_color 
											         4,   #highlight_mode - screen
											        65,   #highlight_opacity
										       "Black",   #shadow_color 
												     3,   #shadow_mode - multiply
											        30,   #shadow_opacity 
												     0,   #surface_contour - linear
											     FALSE,   #use_texture 
									       "Dried mud",   #pattern 
											       100,   #scale 
											       100,   #tex_depth 
											      TRUE,   #invert 
											     FALSE)   #merge	
    
    #Create a text copy
    non_empty = pdb.gimp_edit_copy(textLayer)
    copytextLayer= pdb.gimp_layer_new(img, img.width, img.height, RGB, "Text copy", 0, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_add_layer(img, copytextLayer, 0)
    pdb.gimp_layer_add_alpha(copytextLayer)
    pdb.gimp_drawable_fill(copytextLayer, FILL_TRANSPARENT)
    copytextLayer = pdb.gimp_edit_paste(copytextLayer, FALSE)
    pdb.gimp_floating_sel_anchor(copytextLayer)  
    copytextLayer = pdb.gimp_image_get_layer_by_name(img, "Text copy")
   
	#Text copy inner Glow                           
    pdb.python_layerfx_inner_glow(img, copytextLayer,
								         (140, 0, 0),   #color
								                 100,   #opacity
								                   0,   #contou - linear
								                   0,   #noise
                                                  17,   #mode - burn
                                                   1,   #source
                                                   0,   #choke
                                                  60,   #size
                                               FALSE)   #merge
	
    #Text copy bBevel and emboss
    pdb.python_layerfx_bevel_emboss(img, copytextLayer, 
												     1,   #style inner bevel
											        65,   #depth 
												     0,   #direction 
											        20,   #size 
											        10,   #soften
											        55,   #angle 
											        30,   #altitude 
											         0,   #gloss_contour - linear
										       "White",   #highlight_color 
											         4,   #highlight_mode - screen
											        65,   #highlight_opacity
										       "Black",   #shadow_color 
												     5,   #shadow_mode - overlay
											        20,   #shadow_opacity 
												     0,   #surface_contour - linear
											     FALSE,   #use_texture 
									       "Dried mud",   #pattern 
											       100,   #scale 
											       100,   #tex_depth 
											      TRUE,   #invert 
											     FALSE)   #merge	
	
	#Text copy stroke    
    pdb.python_layerfx_stroke(img, copytextLayer, 
				                       (0, 0, 0),   #color
                                             100,	#opacity
										       0,	#mode - normal
									           3,	#size
									         100,	#position
									       FALSE)	#merge    
	
	#Tex copy pattern overlay		    
    pdb.python_layerfx_pattern_overlay(img, copytextLayer, 
						             "Spiderman Mesh.jpg",   #pattern
								                      100,   #opacity
							                            0,   #mode - normal
								                      180,   #scale
									                    0,   #interpolation
								                    FALSE)   #merge
	
    #Make the text lighter
    textcopypat = pdb.gimp_image_get_layer_by_name(img, "Text copy-pattern")
    pdb.gimp_drawable_hue_saturation(textcopypat, 
                                               0,  #hue_range 
                                               0,  #hue_offset 
                                              25,  #lightness 
                                             100,  #saturation
                                               0)  #overlap
	
    #Spiderman
    spidermanLayer = pdb.gimp_layer_new(img, img.width, img.height, RGB_IMAGE, "Spiderman", 100, LAYER_MODE_NORMAL)
    pdb.gimp_layer_add_alpha(spidermanLayer)
    pdb.gimp_image_add_layer(img, spidermanLayer, 0)
    pdb.gimp_drawable_fill(spidermanLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("Spiderman2")
    pdb.gimp_context_set_brush_size(500)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_force(1)
    pdb.gimp_paintbrush_default(spidermanLayer, 2, [190, 300])
	  
    #Set gimp to default
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
    
register(
    "create_a_web_and_spiderman_text",
    "Creates a Spiderman text",
    "Creates a web an a Spiderman text",
    "Pocholo",
    "Pocholo",
    "2021",
    "Create a Web and Spiderman text",
    "",
    [
    
    (PF_FILENAME, "filename", "Image:", None),
    (PF_STRING, "customtext", "Text string", 'SPIDERMAN'),
    (PF_FONT, "font", "Font name", "Robofan Free"),
    (PF_SPINNER, "size", "Text size", 500, (500, 500, 1)),       
    ],
    [],
    create_a_web_and_spiderman_text, menu="<Image>/Pocholo-scripts/Create a Web and Spiderman text",
    domain=("gimp20", gimp.locale_directory))
main()			